-- Copyright (c) 2015 Cloudera, Inc. All rights reserved.
--
-- Tables for HDFS activity analytics

-- Metadata table for file information
-- There is a staging set of tables to minimize the amount of time the
-- activity analytics are unavailable when loading new data
-- ID columns in the staging tables are automatically generated, but because
-- we do a direct copy of staging to prod tables we don't need the ID columns
-- in prod tables to be auto-generated.

-- *********************
-- * production tables *
-- *********************

-- main fact table
CREATE TABLE NAV_FILE_READS (
  -- service id
  SID         TINYINT NOT NULL,
  -- parent id
  PID         BIGINT NOT NULL,
  -- file id
  FID         BIGINT NOT NULL,
  -- number of read operations for file
  READ_COUNT  BIGINT NOT NULL,
  PRIMARY KEY (SID, FID),
  -- group by directory
  INDEX IDX_FILE_READS_SID_PID (SID, PID),
  -- histogram and range filter on read count for drill down
  INDEX IDX_READS_COUNT (READ_COUNT)
) ENGINE = InnoDB;

-- file metadata
CREATE TABLE NAV_FILE_INFO (
  ID         BIGINT NOT NULL,
  -- Parent directory id
  PID        BIGINT NOT NULL,
  -- Service id
  SID        TINYINT NOT NULL,
  -- Full file path
  PATH       LONGTEXT NOT NULL,
-- PATH md5 for indexing
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  INDEX IDX_FILE_SID_PID_ID (SID, PID),
  UNIQUE INDEX IDX_FILE_SID_PATH (SID, PATH_MD5)
) ENGINE = InnoDB;

-- directory metadata
CREATE TABLE NAV_PARENT_INFO (
  ID         BIGINT NOT NULL,
-- Service id
  SID        TINYINT NOT NULL,
-- Full directory path
  PATH       LONGTEXT NOT NULL,
-- PATH md5 for indexing
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  INDEX IDX_PARENT_SID (SID),
  UNIQUE INDEX IDX_PARENT_SID_PATH_MD5 (SID, PATH_MD5),
  -- directory filter
  INDEX IDX_PARENT_SID_PATH (PATH(255))
) ENGINE = InnoDB;

-- service metadata
CREATE TABLE NAV_SERVICE_INFO (
  ID    TINYINT NOT NULL,
  NAME  VARCHAR(255) NOT NULL,
  PRIMARY KEY (ID),
  UNIQUE INDEX IDX_SERVICE_INFO_NAME (NAME)
) ENGINE = InnoDB;

-- ******************
-- * staging tables *
-- ******************

CREATE TABLE NAV_FILE_READS_STAGING (
  SID         TINYINT NOT NULL,
  PID         BIGINT,
  FID         BIGINT NOT NULL,
  READ_COUNT  BIGINT NOT NULL,
  PRIMARY KEY (SID, FID),
  -- group by directory
  INDEX IDX_FILE_READS_STG_SID_PID (SID, PID),
  INDEX IDX_READS_STG_COUNT (READ_COUNT)
) ENGINE = InnoDB;

CREATE TABLE NAV_FILE_INFO_STAGING (
  ID         BIGINT NOT NULL AUTO_INCREMENT,
  -- Parent directory id, populated post limiting so nullable in stage
  PID        BIGINT,
  -- Service id
  SID        TINYINT NOT NULL,
  -- Full file path
  PATH       LONGTEXT NOT NULL,
  -- PATH MD5
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  INDEX IDX_FILE_STG_SID_PID (SID, PID),
  UNIQUE INDEX IDX_FILE_STG_SID_PATH (SID, PATH_MD5)
) ENGINE = InnoDB;

CREATE TABLE NAV_PARENT_INFO_STAGING (
  ID         BIGINT NOT NULL AUTO_INCREMENT,
  -- Service id
  SID        TINYINT NOT NULL,
  -- Full file path
  PATH       LONGTEXT NOT NULL,
  -- PATH md5 for indexing
  PATH_MD5   CHAR(32),
  PRIMARY KEY (ID),
  INDEX IDX_PARENT_STG_SID (SID),
  UNIQUE INDEX IDX_PARENT_STG_SID_PATH (SID, PATH_MD5)
) ENGINE = InnoDB;

CREATE TABLE NAV_SERVICE_INFO_STAGING (
  ID    TINYINT NOT NULL AUTO_INCREMENT,
  NAME  VARCHAR(255) NOT NULL,
  PRIMARY KEY (ID),
  UNIQUE INDEX IDX_SERVICE_INFO_STG_NAME (NAME)
) ENGINE = InnoDB;

-- Staging table for a single service pre-limiting
CREATE TABLE NAV_COUNT_STAGING_RAW (
  FID         BIGINT NOT NULL,
  READ_COUNT  BIGINT NOT NULL,
  PRIMARY KEY (FID)
) ENGINE = InnoDB;


-- ****************
-- * query tables *
-- ****************

-- Summary info for binning
CREATE TABLE NAV_FILE_READS_SUMMARY (
  SID TINYINT NOT NULL,
  MIN_READ_COUNT BIGINT NOT NULL,
  MAX_READ_COUNT BIGINT NOT NULL
) ENGINE = InnoDB;

-- Histogram does not need file level info
CREATE VIEW NAV_HDFS_ACTIVITY_HIST
AS SELECT si.NAME AS SERVICE_NAME, fr.READ_COUNT,
     su.MIN_READ_COUNT, su.MAX_READ_COUNT
   FROM NAV_FILE_READS fr
     INNER JOIN NAV_SERVICE_INFO si ON fr.SID=si.ID
     INNER JOIN NAV_FILE_READS_SUMMARY su ON fr.SID=su.SID;

-- Drilldown does not need file names
CREATE VIEW NAV_HDFS_ACTIVITY
AS SELECT fr.*, si.NAME AS SERVICE_NAME, pi.PATH AS PARENT
   FROM NAV_FILE_READS fr
     INNER JOIN NAV_SERVICE_INFO si ON fr.SID=si.ID
     INNER JOIN NAV_PARENT_INFO pi ON fr.PID=pi.ID AND pi.SID=si.ID;