-- Copyright (c) 2015 Cloudera, Inc. All rights reserved.
--
-- Tables for HDFS activity analytics

-- *** NOTE *** this requires exec privilege on dbms_crypto for nav user

-- Metadata table for file information
-- There is a staging set of tables to minimize the amount of time the
-- activity analytics are unavailable when loading new data
-- ID columns in the staging tables are automatically generated, but because
-- we do a direct copy of staging to prod tables we don't need the ID columns
-- in prod tables to be auto-generated.

-- *********************
-- * production tables *
-- *********************

CREATE TABLE NAV_FILE_READS (
  -- service id
  SID         NUMBER(3) NOT NULL,
  -- parent id
  PID         NUMBER(19) NOT NULL,
  -- file id
  FID         NUMBER(19) NOT NULL,
  -- number of reads for a given file
  READ_COUNT  NUMBER(19) NOT NULL,
  PRIMARY KEY (SID, FID)
);

-- directory level drill-down
CREATE INDEX IDX_FILE_READS_SID_PID ON NAV_FILE_READS (SID, PID);
-- histogram
CREATE INDEX IDX_READS_COUNT ON NAV_FILE_READS (READ_COUNT);

-- file metadata
CREATE TABLE NAV_FILE_INFO (
  ID         NUMBER(19) NOT NULL,
  -- Parent directory id
  PID        NUMBER(19) NOT NULL,
  -- Service id
  SID        NUMBER(3) NOT NULL,
  -- Full file path
  PATH       CLOB NOT NULL,
  -- PATH md5 for indexing
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_FILE_SID_PATH UNIQUE (SID, PATH_MD5)
);

CREATE INDEX IDX_FILE_SID_PID ON NAV_FILE_INFO (SID, PID);

-- directory metadata
CREATE TABLE NAV_PARENT_INFO (
  ID         NUMBER(19) NOT NULL,
  -- Service id
  SID        NUMBER(3) NOT NULL,
  -- Full directory path
  PATH       CLOB NOT NULL,
  -- PATH md5 for indexing
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_PARENT_SID_PATH UNIQUE (SID, PATH_MD5)
);

-- histogram
CREATE INDEX IDX_PARENT_SID ON NAV_PARENT_INFO (SID);

-- service metadata
CREATE TABLE NAV_SERVICE_INFO (
  ID    NUMBER(3) NOT NULL,
  NAME  VARCHAR2(255) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_SERVICE_INFO_NAME UNIQUE (NAME)
);

-- ******************
-- * staging tables *
-- ******************

CREATE TABLE NAV_FILE_READS_STAGING (
  SID         NUMBER(3) NOT NULL,
  PID         NUMBER(19),
  FID         NUMBER(19) NOT NULL,
  READ_COUNT  NUMBER(19) NOT NULL,
  PRIMARY KEY (SID, FID)
);

CREATE INDEX IDX_READS_STG_SID_PID ON NAV_FILE_READS_STAGING (SID, PID);
CREATE INDEX IDX_READS_STG_COUNT ON NAV_FILE_READS_STAGING (READ_COUNT);

CREATE SEQUENCE NAV_FILE_INFO_STAGING_seq
START WITH 1
INCREMENT BY 1;

CREATE TABLE NAV_FILE_INFO_STAGING (
  ID         NUMBER(19) NOT NULL,
  -- Parent directory id
  PID        NUMBER(19),
  -- Service id
  SID        NUMBER(3) NOT NULL,
  -- Full file path
  PATH       CLOB NOT NULL,
  -- PATH MD5
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_FILE_STG_SID_PATH UNIQUE (SID, PATH_MD5)
);

CREATE INDEX IDX_FILE_STG_SID_PID ON NAV_FILE_INFO_STAGING (SID, PID);

CREATE SEQUENCE NAV_PARENT_INFO_STAGING_seq
START WITH 1
INCREMENT BY 1;

CREATE TABLE NAV_PARENT_INFO_STAGING (
  ID         NUMBER(19) NOT NULL,
  -- Service id
  SID        NUMBER(3) NOT NULL,
  -- Full file path
  PATH       CLOB NOT NULL,
  -- PATH md5 for indexing
  PATH_MD5   CHAR(32),
  PRIMARY KEY (ID),
  CONSTRAINT IDX_PARENT_STG_SID_PATH UNIQUE (SID, PATH_MD5)
);

CREATE INDEX IDX_PARENT_STG_SID ON NAV_PARENT_INFO_STAGING (SID);

CREATE SEQUENCE NAV_SERVICE_INFO_STAGING_seq
START WITH 1
INCREMENT BY 1;

CREATE TABLE NAV_SERVICE_INFO_STAGING (
  ID    NUMBER(3) NOT NULL,
  NAME  VARCHAR2(255) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_SERVICE_INFO_STG_NAME UNIQUE (NAME)
);

-- Single service staging, pre-limiting
CREATE TABLE NAV_COUNT_STAGING_RAW (
  PATH_MD5    CHAR(32) NOT NULL,
  READ_COUNT  NUMBER(19) NOT NULL,
  PRIMARY KEY (PATH_MD5)
);


-- ****************
-- * query tables *
-- ****************

-- Summary info for binning
CREATE TABLE NAV_FILE_READS_SUMMARY (
  SID NUMBER(3) NOT NULL,
  MIN_READ_COUNT NUMBER(19) NOT NULL,
  MAX_READ_COUNT NUMBER(19) NOT NULL
);

-- Histogram do not need file or directory metadata
CREATE VIEW NAV_HDFS_ACTIVITY_HIST
AS SELECT si.NAME AS SERVICE_NAME, fr.READ_COUNT,
     su.MIN_READ_COUNT, su.MAX_READ_COUNT
   FROM NAV_FILE_READS fr
     INNER JOIN NAV_SERVICE_INFO si ON fr.SID=si.ID
     INNER JOIN NAV_FILE_READS_SUMMARY su ON fr.SID=su.SID;

-- drilldown only need directory path
CREATE VIEW NAV_HDFS_ACTIVITY
AS SELECT fr.*, si.NAME AS SERVICE_NAME, pi.PATH AS PARENT
   FROM NAV_FILE_READS fr
     INNER JOIN NAV_SERVICE_INFO si ON fr.SID=si.ID
     INNER JOIN NAV_PARENT_INFO pi ON fr.PID=pi.ID AND pi.SID=si.ID;