-- Copyright (c) 2015 Cloudera, Inc. All rights reserved.
--
-- Tables for HDFS activity analytics

-- Metadata table for file information
-- There is a staging set of tables to minimize the amount of time the
-- activity analytics are unavailable when loading new data
-- ID columns in the staging tables are automatically generated, but because
-- we do a direct copy of staging to prod tables we don't need the ID columns
-- in prod tables to be auto-generated.

-- *********************
-- * production tables *
-- *********************

-- main fact table
CREATE TABLE NAV_FILE_READS (
  -- service id
  SID         SMALLINT NOT NULL,
  -- parent id
  PID         BIGINT NOT NULL,
  -- file id
  FID         BIGINT NOT NULL,
  -- number of reads
  READ_COUNT  BIGINT NOT NULL,
  PRIMARY KEY (SID, FID)
);

-- group by directory
CREATE INDEX IDX_FILE_READS_SID_PID ON NAV_FILE_READS (SID, PID);
-- histogram
CREATE INDEX IDX_READS_COUNT ON NAV_FILE_READS (READ_COUNT);

-- file metadata
CREATE TABLE NAV_FILE_INFO (
  ID         BIGINT NOT NULL,
  -- Parent directory id
  PID        BIGINT NOT NULL,
  -- Service id
  SID        SMALLINT NOT NULL,
  -- Full file path
  PATH       TEXT NOT NULL,
  -- PATH md5 for indexing
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_FILE_SID_PID_ID UNIQUE (SID, PID, ID),
  CONSTRAINT IDX_FILE_SID_PATH UNIQUE (SID, PATH_MD5)
);

-- directory metadata
CREATE TABLE NAV_PARENT_INFO (
  ID         BIGINT NOT NULL,
  -- Service id
  SID        SMALLINT NOT NULL,
  -- Full directory
  PATH       TEXT NOT NULL,
  -- PATH md5 for indexing
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_PARENT_SID_PATH_MD5 UNIQUE (SID, PATH_MD5)
);

CREATE INDEX IDX_PARENT_SID ON NAV_PARENT_INFO (SID);
-- directory filter (pattern searches)
CREATE INDEX IDX_PARENT_SID_PATH ON NAV_PARENT_INFO (PATH text_pattern_ops);

-- service metadata
CREATE TABLE NAV_SERVICE_INFO (
  ID    SMALLINT NOT NULL,
  NAME  VARCHAR(255) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_SERVICE_INFO_NAME UNIQUE (NAME)
);

-- ******************
-- * staging tables *
-- ******************

CREATE TABLE NAV_FILE_READS_STAGING (
  SID         SMALLINT NOT NULL,
  PID         BIGINT,
  FID         BIGINT NOT NULL,
  READ_COUNT  BIGINT NOT NULL,
  PRIMARY KEY (SID, FID)
);

CREATE INDEX IDX_READS_STG_SID_PID ON NAV_FILE_READS_STAGING (SID, PID);
CREATE INDEX IDX_READS_STG_COUNT ON NAV_FILE_READS_STAGING (READ_COUNT);

CREATE SEQUENCE NAV_FILE_INFO_STAGING_seq
START WITH 1
INCREMENT BY 1;

CREATE TABLE NAV_FILE_INFO_STAGING (
  ID         BIGINT NOT NULL DEFAULT NEXTVAL ('NAV_FILE_INFO_STAGING_seq'),
  -- Parent directory
  PID        BIGINT,
  -- Service id
  SID        SMALLINT NOT NULL,
  -- Full file path
  PATH       TEXT NOT NULL,
  -- PATH MD5
  PATH_MD5   CHAR(32) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_FILE_STG_SID_PID_ID UNIQUE (SID, PID, ID),
  CONSTRAINT IDX_FILE_STG_SID_PATH UNIQUE (SID, PATH_MD5)
);

CREATE SEQUENCE NAV_PARENT_INFO_STAGING_seq
START WITH 1
INCREMENT BY 1;

CREATE TABLE NAV_PARENT_INFO_STAGING (
  ID         BIGINT NOT NULL DEFAULT NEXTVAL ('NAV_PARENT_INFO_STAGING_seq'),
  -- Service id
  SID        SMALLINT NOT NULL,
  -- Full file path
  PATH       TEXT NOT NULL,
  -- PATH md5 for indexing
  PATH_MD5   CHAR(32),
  PRIMARY KEY (ID),
  CONSTRAINT IDX_PARENT_STG_SID_PATH UNIQUE (SID, PATH_MD5)
);

CREATE INDEX IDX_PARENT_STG_SID ON NAV_PARENT_INFO_STAGING (SID);

CREATE SEQUENCE NAV_SERVICE_INFO_STAGING_seq
START WITH 1
INCREMENT BY 1;

CREATE TABLE NAV_SERVICE_INFO_STAGING (
  ID    SMALLINT NOT NULL DEFAULT NEXTVAL ('NAV_SERVICE_INFO_STAGING_seq'),
  NAME  VARCHAR(255) NOT NULL,
  PRIMARY KEY (ID),
  CONSTRAINT IDX_SERVICE_INFO_STG_NAME UNIQUE (NAME)
);

-- Staging table for a given service
CREATE TABLE NAV_COUNT_STAGING_RAW (
  FID         BIGINT NOT NULL,
  READ_COUNT  BIGINT NOT NULL,
  PRIMARY KEY (FID)
);


-- ****************
-- * query tables *
-- ****************

-- Summary info for binning
CREATE TABLE NAV_FILE_READS_SUMMARY (
  SID SMALLINT NOT NULL,
  MIN_READ_COUNT BIGINT NOT NULL,
  MAX_READ_COUNT BIGINT NOT NULL
);

-- Histogram do not need file or directory metadata
CREATE VIEW NAV_HDFS_ACTIVITY_HIST
AS SELECT si.NAME AS SERVICE_NAME, fr.READ_COUNT,
     su.MIN_READ_COUNT, su.MAX_READ_COUNT
   FROM NAV_FILE_READS fr
     INNER JOIN NAV_SERVICE_INFO si ON fr.SID=si.ID
     INNER JOIN NAV_FILE_READS_SUMMARY su ON fr.SID=su.SID;

-- drilldown only need directory path
CREATE VIEW NAV_HDFS_ACTIVITY
AS SELECT fr.*, si.NAME AS SERVICE_NAME, pi.PATH AS PARENT
   FROM NAV_FILE_READS fr
     INNER JOIN NAV_SERVICE_INFO si ON fr.SID=si.ID
     INNER JOIN NAV_PARENT_INFO pi ON fr.PID=pi.ID AND pi.SID=si.ID;